@@ -554,8 +554,10 @@ _gst_get_cur_string (void)
 
       /* Copy back to the beginning of the buffer to save memory.
          TODO: might want to do so at the first overflow instead.  */
-      size = in_stream->st_oop.end - in_stream->st_oop.ptr + 1;
-      memmove (in_stream->st_oop.buf, in_stream->st_oop.ptr, size);
+      size = in_stream->st_oop.end - in_stream->st_oop.ptr;
+      if (size)
+	memmove (in_stream->st_oop.buf, in_stream->st_oop.ptr, size);
+      in_stream->st_oop.buf[size] = 0;
       in_stream->fileOffset += in_stream->st_oop.ptr - in_stream->st_oop.buf;
       in_stream->st_oop.ptr = in_stream->st_oop.buf;
       in_stream->st_oop.end = in_stream->st_oop.buf + size;
