@@ -7,7 +7,7 @@
 
 /***********************************************************************
  *
- * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002
+ * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2006
  * Free Software Foundation, Inc.
  * Written by Steve Byrne.
  *
@@ -18,6 +18,28 @@
  * Software Foundation; either version 2, or (at your option) any later 
  * version.
  * 
+ * Linking GNU Smalltalk statically or dynamically with other modules is
+ * making a combined work based on GNU Smalltalk.  Thus, the terms and
+ * conditions of the GNU General Public License cover the whole
+ * combination.
+ *
+ * In addition, as a special exception, the Free Software Foundation
+ * give you permission to combine GNU Smalltalk with free software
+ * programs or libraries that are released under the GNU LGPL and with
+ * independent programs running under the GNU Smalltalk virtual machine.
+ *
+ * You may copy and distribute such a system following the terms of the
+ * GNU GPL for GNU Smalltalk and the licenses of the other code
+ * concerned, provided that you include the source code of that other
+ * code when and as the GNU GPL requires distribution of source code.
+ *
+ * Note that people who make modified versions of GNU Smalltalk are not
+ * obligated to grant this special exception for their modified
+ * versions; it is their choice whether to do so.  The GNU General
+ * Public License gives permission to release a modified version without
+ * this exception; this exception also makes it possible to release a
+ * modified version which carries forward this exception.
+ *
  * GNU Smalltalk is distributed in the hope that it will be useful, but WITHOUT
  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
  * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
@@ -57,7 +79,9 @@ enum
   PRIM_SYNC_POLL,               /* read/write/exception */
   PRIM_ASYNC_POLL,              /* operation:semaphore: */
   PRIM_IS_PIPE,                 /* no args */
-  PRIM_MK_TEMP                  /* base: */
+  PRIM_MK_TEMP,                 /* base: */
+  PRIM_GET_CHARS_AT,            /* data:from:to:absOfs: */
+  PRIM_PUT_CHARS_AT             /* data:from:to:absOfs: */
 };
 
 /* These macros are used to quickly compute the number of words needed
@@ -518,14 +542,6 @@ prim_table_entry;
 #define PRIM_RETURN_SMALL_INTEGER	0x0100	/* 31 or 63 bits */
 #define PRIM_RETURN_SMALL_SMALLINTEGER	0x0300	/* 30 or 62 bits */
 
-/* This is the bridge to the primitive operations in the GNU Smalltalk
-   system.  This function invokes the proper primitive_func with the
-   correct id and the same NUMARGS and METHODOOP with which it was
-   invoked.  */
-extern intptr_t _gst_execute_primitive_operation (int primitive,
-					          volatile int numArgs) 
-  ATTRIBUTE_HIDDEN;
-
 /* This can be used to obtain information on a particular primitive
    operations in the GNU Smalltalk system.  */
 extern prim_table_entry * _gst_get_primitive_attributes (int primitive)
