@@ -140,7 +140,14 @@ static int char_literal (int c,
 
 /* Parse a binary operator.  C is the first symbol in the selector */
 static int scan_bin_op (int c,
-			 YYSTYPE * lvalp);
+			YYSTYPE * lvalp);
+
+/* Actual work for scan_bin_op is done here.  MAYBE_NUMBER is false if
+   we cannot parse a negative number in this context.  */ 
+static int scan_bin_op_1 (int c,
+			  YYSTYPE * lvalp,
+			  mst_Boolean maybe_number);
+
 
 /* Parse a string literal.  C is '\'' */
 static int string_literal (int c,
@@ -487,7 +494,7 @@ scan_symbol (int c,
   /* We can read a binary operator and return a SYMBOL_LITERAL,... */
   if (CHAR_TAB (ic)->char_class & BIN_OP_CHAR)
     {
-      scan_bin_op (ic, lvalp);
+      scan_bin_op_1 (ic, lvalp, false);
       return SYMBOL_LITERAL;
     }
 
@@ -519,8 +526,9 @@ scan_symbol (int c,
 
 
 int
-scan_bin_op (int c,
-	      YYSTYPE *lvalp)
+scan_bin_op_1 (int c,
+	       YYSTYPE *lvalp,
+	       mst_Boolean maybe_number)
 {
   char buf[3];
   int ic;
@@ -551,7 +559,7 @@ scan_bin_op (int c,
 
       /* We come here also for a negative number, which we handle
          specially.  */
-      if (c == '-' && is_digit (ic))
+      if (maybe_number && c == '-' && is_digit (ic))
 	return (scan_number ('-', lvalp));
 
       buf[1] = 0;
@@ -567,6 +575,13 @@ scan_bin_op (int c,
     return (BINOP);
 }
 
+int
+scan_bin_op (int c,
+	     YYSTYPE *lvalp)
+{
+  return scan_bin_op_1 (c, lvalp, true);
+}
+
 int
 string_literal (int c,
 		YYSTYPE * lvalp)
@@ -801,8 +816,9 @@ scan_number (int c,
 		(num * (10 raisedToInteger: exponent)
 		   asScaledDecimal: floatExponent) */
 	lvalp->oval =
-	  _gst_msg_send (numOOP, _gst_as_scaled_decimal_scale_symbol,
+	  _gst_msg_send (numOOP, _gst_as_scaled_decimal_radix_scale_symbol,
 			 FROM_INT (exponent),
+			 FROM_INT (base),
 			 FROM_INT ((int) floatExponent), 
 			 NULL);
 
