@@ -7,7 +7,7 @@
 
 /***********************************************************************
  *
- * Copyright 2000, 2001, 2002, 2003 Free Software Foundation, Inc.
+ * Copyright 2000, 2001, 2002, 2003, 2006 Free Software Foundation, Inc.
  * Written by Paolo Bonzini.
  *
  * This file is part of GNU Smalltalk.
@@ -17,6 +17,28 @@
  * Software Foundation; either version 2, or (at your option) any later
  * version.
  *
+ * Linking GNU Smalltalk statically or dynamically with other modules is
+ * making a combined work based on GNU Smalltalk.  Thus, the terms and
+ * conditions of the GNU General Public License cover the whole
+ * combination.
+ *
+ * In addition, as a special exception, the Free Software Foundation
+ * give you permission to combine GNU Smalltalk with free software
+ * programs or libraries that are released under the GNU LGPL and with
+ * independent programs running under the GNU Smalltalk virtual machine.
+ *
+ * You may copy and distribute such a system following the terms of the
+ * GNU GPL for GNU Smalltalk and the licenses of the other code
+ * concerned, provided that you include the source code of that other
+ * code when and as the GNU GPL requires distribution of source code.
+ *
+ * Note that people who make modified versions of GNU Smalltalk are not
+ * obligated to grant this special exception for their modified
+ * versions; it is their choice whether to do so.  The GNU General
+ * Public License gives permission to release a modified version without
+ * this exception; this exception also makes it possible to release a
+ * modified version which carries forward this exception.
+ *
  * GNU Smalltalk is distributed in the hope that it will be useful, but WITHOUT
  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
@@ -30,6 +52,7 @@
 
 
 #include "gstpriv.h"
+#include "match.h"
 
 /* Define this to have verbose messages from the JIT compiler's
    basic-block split phase.  */
@@ -909,15 +932,19 @@ _gst_compute_stack_positions (gst_uchar * bp,
             MAKE_DIRTY_BLOCK { }
 
             SEND {
-	      balance += -num_args;
+	      balance -= super + num_args;
 	    }
 
-	    SEND_ARITH, SEND_IMMEDIATE {
-	      balance += -_gst_builtin_selectors[n]->numArgs;
+	    SEND_ARITH {
+	      balance -= _gst_builtin_selectors[n]->numArgs;
+	    }
+
+	    SEND_IMMEDIATE {
+	      balance -= super + _gst_builtin_selectors[n]->numArgs;
 	    }
 
             SEND_SPECIAL {
-	      balance += -_gst_builtin_selectors[n + 16]->numArgs;
+	      balance -= _gst_builtin_selectors[n + 16]->numArgs;
 	    }
 
             INVALID {
@@ -1088,7 +1115,7 @@ typedef struct partially_constructed_array {
   /* Todo: recurse into BlockClosures! */ \
   last_used_literal = literals[n]; \
   if ((n) >= num_literals) \
-    return (_gst_debug(), "literal out of range");
+    return ("literal out of range");
 
 #define CHECK_TEMP(n) \
   last_used_literal = NULL; \
@@ -1111,10 +1138,10 @@ typedef struct partially_constructed_array {
     return ("Invalid global variable access");
 
 #define LIT_VARIABLE_CLASS(n) \
-  /* Special case Array because it is used to compile {...} */ \
-  (ASSOCIATION_VALUE (literals[(n)]) == _gst_array_class \
-    ? OOP_CLASS (_gst_array_class) \
-    : FROM_INT (VARYING))
+  /* Special case classes because of super and {...} */ \
+  (IS_A_CLASS (ASSOCIATION_VALUE (literals[(n)])) \
+   ? OOP_CLASS (ASSOCIATION_VALUE (literals[(n)])) \
+   : FROM_INT (VARYING))
 
 #define LITERAL_CLASS(n) \
   OOP_INT_CLASS (literals[(n)])
@@ -1357,7 +1384,7 @@ _gst_verify_method (OOP methodOOP, int *num_outer_temps, int depth)
             MAKE_DIRTY_BLOCK { }
 
             SEND {
-	      balance += -num_args;
+	      balance -= super + num_args;
 
 	      /* Sends touch the new stack top, so they require an extra slot.  */
 	      if (curr_sp + balance < 1)
@@ -1368,7 +1395,7 @@ _gst_verify_method (OOP methodOOP, int *num_outer_temps, int depth)
 	      if (!_gst_builtin_selectors[n])
 		return ("invalid immediate send");
 
-	      balance += -_gst_builtin_selectors[n]->numArgs;
+	      balance -= _gst_builtin_selectors[n]->numArgs;
 
 	      /* Sends touch the new stack top, so they require an extra slot.  */
 	      if (curr_sp + balance < 1)
@@ -1379,7 +1406,7 @@ _gst_verify_method (OOP methodOOP, int *num_outer_temps, int depth)
 	      if (!_gst_builtin_selectors[n + 16])
 		return ("invalid immediate send");
 
-	      balance += -_gst_builtin_selectors[n + 16]->numArgs;
+	      balance -= _gst_builtin_selectors[n + 16]->numArgs;
 
 	      /* Sends touch the new stack top, so they require an extra slot.  */
 	      if (curr_sp + balance < 1)
@@ -1390,7 +1417,7 @@ _gst_verify_method (OOP methodOOP, int *num_outer_temps, int depth)
 	      if (!_gst_builtin_selectors[n])
 		return ("invalid immediate send");
 
-	      balance += -_gst_builtin_selectors[n]->numArgs;
+	      balance -= super + _gst_builtin_selectors[n]->numArgs;
 
 	      /* Sends touch the new stack top, so they require an extra slot.  */
 	      if (curr_sp + balance < 1)
@@ -1613,8 +1640,15 @@ _gst_verify_method (OOP methodOOP, int *num_outer_temps, int depth)
 	    }
 
 	    SEND {
+	      if (super
+		  && (!last_used_literal
+		      || (!IS_A_CLASS (last_used_literal)
+		          && !IS_A_METACLASS (last_used_literal))
+		      || !is_a_kind_of (methodClass, last_used_literal)))
+		return ("Invalid send to super");
+
 	      last_used_literal = NULL;
-	      sp -= num_args;
+	      sp -= super + num_args;
 	      if (super && sp[-1] != FROM_INT (SELF))
 		return ("Invalid send to super");
 
@@ -1703,6 +1737,7 @@ _gst_verify_method (OOP methodOOP, int *num_outer_temps, int depth)
 	    SEND_IMMEDIATE {
 	      if (n == NEW_COLON_SPECIAL
 		  && IS_INT (last_used_literal)
+		  && last_used_literal != FROM_INT (0)
 		  && sp[-2] == OOP_CLASS (_gst_array_class))
 		{
 		  partially_constructed_array *a;
@@ -1726,7 +1761,18 @@ _gst_verify_method (OOP methodOOP, int *num_outer_temps, int depth)
 		}
 	      else
 	        {
-	          sp -= _gst_builtin_selectors[n]->numArgs;
+		  if (super
+		      && (!last_used_literal
+			  || (!IS_A_CLASS (last_used_literal)
+			      && !IS_A_METACLASS (last_used_literal))
+			  || !is_a_kind_of (methodClass, last_used_literal)))
+		    return (_gst_debug (), "Invalid send to super");
+
+		  last_used_literal = NULL;
+	          sp -= super + _gst_builtin_selectors[n]->numArgs;
+	          if (super && sp[-1] != FROM_INT (SELF))
+		    return ("Invalid send to super");
+
 	          sp[-1] = FROM_INT (VARYING);
 		}
 	    }
